﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;

using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using System.Collections;

using SampleDataGeneratorNS;
using Infragistics.Olap.Data;
using Infragistics.Controls.Grids.Primitives;
using System.Collections.Specialized;
using System.Data;
using System.Reflection;
using System.Xml;
using System.IO;

namespace RockyCube
{
    public partial class MainPage : UserControl
    {
        public class RockyProperty
        {
            public RockyProperty(String name, Type type)
            {
                this.name = name;
                this.type = type;
            }
            public String name { set; get; }
            public Type type { set; get; }
        }

        public List<RockyProperty> allProps = new List<RockyProperty>();

        public MainPage()
        {
            InitializeComponent();

            Loaded += new RoutedEventHandler(DrillDown_Loaded);

            allProps.Add(new RockyProperty("forecastGroup", typeof(string)));
            allProps.Add(new RockyProperty("itemNum", typeof(string)));
            allProps.Add(new RockyProperty("itemLoc", typeof(string)));
            allProps.Add(new RockyProperty("vendorID", typeof(string)));
            allProps.Add(new RockyProperty("Date", typeof(DateTime)));
            allProps.Add(new RockyProperty("Class", typeof(string)));
            allProps.Add(new RockyProperty("BuyerID", typeof(string)));
            allProps.Add(new RockyProperty("ProductGroup", typeof(string)));
            allProps.Add(new RockyProperty("OrderPolicy", typeof(string)));
            allProps.Add(new RockyProperty("BoMLevel", typeof(string)));
            allProps.Add(new RockyProperty("FCUnits", typeof(double)));
            allProps.Add(new RockyProperty("Actuals", typeof(int)));
            allProps.Add(new RockyProperty("BF_UnitCost", typeof(double)));
            allProps.Add(new RockyProperty("BF_AUSP", typeof(double)));
            allProps.Add(new RockyProperty("BF_Units", typeof(double)));
            allProps.Add(new RockyProperty("CostofSales", typeof(double)));
            allProps.Add(new RockyProperty("GrossRevenue", typeof(double)));
            allProps.Add(new RockyProperty("OpenOrders", typeof(int)));
            allProps.Add(new RockyProperty("BudgetUnits", typeof(int)));
            allProps.Add(new RockyProperty("ShipUnits", typeof(int)));
            allProps.Add(new RockyProperty("BudgetPrice", typeof(double)));
            allProps.Add(new RockyProperty("FcstPrice", typeof(double)));

        }

        void DrillDown_Loaded(object sender, RoutedEventArgs e)
        {
            // Make an HTTP call to load the XML file
            WebClient xmlClient = new WebClient();
            xmlClient.DownloadStringCompleted += new DownloadStringCompletedEventHandler(XMLFileLoaded);
            xmlClient.DownloadStringAsync(new Uri("basecube.xml", UriKind.RelativeOrAbsolute));
        }

        public void XMLFileLoaded(object sender, DownloadStringCompletedEventArgs e)
        {
            string xmlData = null;

            if (e.Error == null)
            {
                xmlData = e.Result;
            }
            else
            {
                MessageBox.Show("ERROR: ***" + e.Error.InnerException.ToString() + "***");
            }

            IList<DynamicTypePropertyInfo> properties = new List<DynamicTypePropertyInfo>();

            foreach (RockyProperty prop in allProps)
            {
                // My version of the code below
                DynamicTypePropertyInfo propInfo = new DynamicTypePropertyInfo
                {
                    PropertyName = prop.name,
                    PropertyType = prop.type
                };
                properties.Add(propInfo);
            }

            DynamicTypeBuilder typeBuilder = new DynamicTypeBuilder
            {
                DynamicAssemblyName = "MyAssembly",
                DynamicTypeName = "Pane"
            };

            Type dynamicType = typeBuilder.GenerateType(properties);

            Type listType = typeof(List<>);
            Type genericListType = listType.MakeGenericType(dynamicType);
            IList list = (IList)Activator.CreateInstance(genericListType);

            FlatDataSource flatDataSource = new FlatDataSource()
            {
                ItemsSource = list,
                Cube = DataSourceBase.GenerateInitialCube("Pane"),
                // if you know the names of demensions you want to be in rows and columns 
                // you can define them here
                Columns = DataSourceBase.GenerateInitialItems("[Columns]"),
                Rows = DataSourceBase.GenerateInitialItems("[Row]"),
                Measures = DataSourceBase.GenerateInitialItems("Value")
            };


            try
            {
                // Create an XmlReader
                using (XmlReader xmlnode = XmlReader.Create(new StringReader(xmlData)))
                {
                    // Parse the file and display each of the nodes.
                    while (xmlnode.Read())
                    {
                        if (xmlnode.NodeType == XmlNodeType.Element && xmlnode.Name == "row")
                        {
                            object myDynamicInstance = Activator.CreateInstance(dynamicType);

                            foreach (RockyProperty prop in allProps)
                            {
                                PropertyInfo propertyVal = dynamicType.GetProperty(prop.name);
                                propertyVal.SetValue(myDynamicInstance, ConvertFromString(prop.name,prop.type, xmlnode[prop.name]), null);
                            }

                            list.Add(myDynamicInstance);
                        }
                    }
                }

                // Set up aggregator
                CubeMetadata cubeMetadata = new CubeMetadata { DataTypeFullName = dynamicType.FullName, DisplayName = "Rocky Cube" };

                DimensionMetadata calcMeasureMetadata = new DimensionMetadata
                {
                    DisplayFormat = "{0:N2}",
                    SourcePropertyName = "BF_AUSP",
                    DimensionType = DimensionType.Measure,
                    DisplayName = "BF_AUSP",
                    Aggregator = new CustomMeasureAggregator(flatDataSource)
                };

                cubeMetadata.DimensionSettings.Add(calcMeasureMetadata);

                flatDataSource.CubesSettings.Add(cubeMetadata);

                xamPivotGrid1.DataSource = flatDataSource;
                xamPivotDataSelector1.DataSource = flatDataSource;
                xamPivotGrid1.DataSource.Columns.CollectionChanged += new NotifyCollectionChangedEventHandler(Columns_CollectionChanged);



                /*
                HierarchyDescriptor regionRepDescriptor = new HierarchyDescriptor { SourcePropertyName = "Region" };
                // this will add a single root element
                regionRepDescriptor.AddLevel("All Values", "All Values");

                HierarchyLevelDescriptor regionLevel =
                new HierarchyLevelDescriptor { LevelExpressionPath = "Region", LevelName = "Regions" };

                HierarchyLevelDescriptor cityLevel =
                new HierarchyLevelDescriptor { LevelExpressionPath = "City", LevelName = "Cities" };

                regionRepDescriptor.LevelDescriptors.Add(regionLevel);
                regionRepDescriptor.LevelDescriptors.Add(cityLevel);

                flatDataSource.HierarchyDescriptors.Add(regionRepDescriptor);
            
                ---

                IEnumerable sampleData = SampleDataGenerator.GenerateSales(500);

                //(xamPivotGrid1.DataSource as FlatDataSource).ItemsSource = sampleData;
            
                FlatDataSource flatDataSource = new FlatDataSource()
                {
                    //Data is the IEnumerable to be used as a source
                    ItemsSource = sampleData,
                    Cube = DataSourceBase.GenerateInitialCube("Sale"),
                    Rows = DataSourceBase.GenerateInitialItems("[Date]"),
                    Columns = DataSourceBase.GenerateInitialItems("[City]"),
                    Measures = DataSourceBase.GenerateInitialItems("AmountOfSale")
                };

                flatDataSource.CubesSettings.Add(cubeMetadata);

                DimensionMetadata numberOfUnitsMetadata = new DimensionMetadata
                {
                    SourcePropertyName = "AmountOfSale",
                    DisplayName = "Amount Of Sale (Avg)",
                    DimensionType = DimensionType.Measure,
                    AggregatorType = AggregatorType.Average,
                    DisplayFormat = "{0:0.##}"
                };

                cubeMetadata.DimensionSettings.Add(numberOfUnitsMetadata);

                CubeMetadata cubeMetadata = new CubeMetadata();
                cubeMetadata.DisplayName = "Sales Data";
                cubeMetadata.DataTypeFullName = typeof(Sale).FullName;
                cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
                {
                    SourcePropertyName = "AmountOfSale",
                    DisplayName = "Amount of sale",
                    DisplayFormat = "{0:C2}"
                });

                cubeMetadata.DimensionSettings.Add(new DimensionMetadata()
                {
                    SourcePropertyName = "NumberOfUnits",
                    DisplayName = "Units sold"
                });

                */

            }
            catch (Exception e1)
            {
                MessageBox.Show(e1.ToString());
            }
        }

        public static object ConvertFromString(string prop, Type type, string value)
        {
            try
            {
                if (type.Name == "String")
                {
                    if (value == null)
                    {
                        return "";
                    }

                    return value;
                }

                if (type.Name == "DateTime")
                {
                    if (value == null)
                    {
                        return new DateTime();
                    }

                    string day   = value.Substring(0, 2);
                    string month = value.Substring(3, 2);
                    string year  = value.Substring(6, 2);

                    return DateTime.Parse(month+"/"+day+"/"+year);
                }

                if (type.Name == "Int32")
                {
                    if (value == null)
                    {
                        return 0;
                    }

                    return Int32.Parse(value);
                }

                if (type.Name == "Double")
                {
                    if (value == null)
                    {
                        return 0.0;
                    }

                    return Double.Parse(value);
                }

                // Find the Parse method
                MethodInfo parser = type.GetMethod("Parse", BindingFlags.Static);

                if (parser == null)
                    throw new InvalidOperationException("No static Parse() method in type " + type.Name);

                // Create the instance
                return parser.Invoke(null, new object[] { value });
            }
            catch (FormatException e)
            {
                throw new FormatException("Couldn't parse property "+prop+" value "+value+" as type "+type);
            }
        }


        void DataSource_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
        }

        static LinearGradientBrush defaultFilterColor;

        void Columns_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                FilterViewModelBase obj = e.NewItems[0] as FilterViewModelBase;

                if (obj != null)
                {
                    foreach (IFilterMember filterMember in obj.FilterMembers)
                    {
                        filterMember.PropertyChanged += (s, args) =>
                        {
                            if (args.PropertyName == "IsSelected")
                            {
                                // Find the index of the column on which the event occurred...
                                int index = -1;
                                IFilterMember filter = s as IFilterMember;

                                foreach (Object maybeFilter in this.xamPivotGrid1.DataSource.Columns)
                                {
                                    index++;

                                    if (maybeFilter is IFilterViewModel)
                                    {
                                        IFilterViewModel fvm = (IFilterViewModel)maybeFilter;

                                        if (filter.FilterViewModel == fvm)
                                        {
                                            break;
                                        }
                                    }
                                }

                                // Determine if any of the filter members for the column are active
                                bool foundActiveFilter = false;

                                if (filter.FilterViewModel.FilterMembers != null)
                                {
                                    foreach (IFilterMember member in filter.FilterViewModel.FilterMembers)
                                    {
                                        // If any checkbox is "unchecked" (not selected), 
                                        // this is an "active" filter
                                        if (!(bool)member.IsSelected)
                                        {
                                            foundActiveFilter = true;
                                        }
                                    }
                                }

                                // If we found an active filter
                                if (foundActiveFilter)
                                {
                                    // Turn the column orange
                                    FieldItemControl item = (FieldItemControl)this.xamPivotGrid1.ColumnsDropControl.ItemContainerGenerator.ContainerFromIndex(index);

                                    if (defaultFilterColor == null)
                                        defaultFilterColor = (LinearGradientBrush)item.Background;

                                    item.Background = new SolidColorBrush(Colors.Orange);

                                }
                                else
                                {
                                    // Turn the column back to what it was
                                    FieldItemControl item = (FieldItemControl)this.xamPivotGrid1.ColumnsDropControl.ItemContainerGenerator.ContainerFromIndex(index);
                                    item.Background = defaultFilterColor;
                                }
                            }
                        };

                    }
                }
            }
        }

    }
}
